#include <stdlib.h>
#include <stdio.h>
#include "prover.h"
#include "uthash.h"
#include "tree.h"
#include "symbol_table.h"

extern tnode *create_tnode(int type, int id, tnode *left, tnode *right);
extern prop_node *insert_p_node (char *name);
extern void print_tree(tnode *s);
extern struct prop_node *propsname;
extern struct prop_node *propsid;

tnode *get_fnb(tnode *s, int value, int theta) {
    if (s == NULL)
        return NULL;
    else switch (s->type) {
        case CONSTANT:
            {
                if(value == 0) {
                    if(s->id == CTRUE) {
                        s = create_tnode(CONSTANT,CFALSE,NULL,NULL);
                    } else {
                        s = create_tnode(CONSTANT,CTRUE,NULL,NULL);
                    }
                } else {
                    s = create_tnode(CONSTANT,s->id,NULL,NULL);
                }
                return s;
            }
            break;
        case PROP:
            {
                if (theta == 1) {
                    prop_node *p1, *p2;
                    int id = s->id;
                    HASH_FIND(hid,propsid,&id,sizeof(int),p1);
                    int len = snprintf(NULL, 0, "THETA(%s)",p1->name) + 1;
                    char *name = (char *) malloc(len); 
                    len = snprintf(name, len, "THETA(%s)",p1->name);
                    p2 = insert_p_node(name);
                    s = create_tnode(PROP,p2->id,NULL,NULL);
                    s->value = value;
                    s->theta = theta;
                    return s;
                }
                else {
                    s = create_tnode(PROP,s->id,NULL,NULL);
                    s->value = value;
                    return s;
                }
            }
            break;
        case NEGATION:
            { // neg (p) = F: (p)
                tnode *aux;
                
                aux = s->left;
                if(value == 1)
                    aux = get_fnb(aux, 0, theta);
                else
                    aux = get_fnb(aux, 1, theta);
                return aux;
            }
            break;
        case IMPLICATION:
            {
                if (value == 0) {
                    if (theta == 0) { // F: (p then q) = (T:p and F:q) or (T:theta(p) and F:theta(q))
                        tnode *aux;
                        
                        aux = create_tnode(DISJUNCTION,DISJUNCTION,NULL,NULL);
                        aux->left = create_tnode(CONJUNCTION,CONJUNCTION,s->left,s->right);
                        aux->right = create_tnode(CONJUNCTION,CONJUNCTION,s->left,s->right);
                        
                        aux->left->left = get_fnb(aux->left->left,1,0);
                        aux->left->right = get_fnb(aux->left->right,0,0);
                        aux->right->left = get_fnb(aux->right->left,1,1);
                        aux->right->right = get_fnb(aux->right->right,0,1);
                        
                        return aux;
                    } else { // F: theta(p then q) = (T:p and F:theta(q))
                        tnode *aux;
                        
                        aux = create_tnode(CONJUNCTION,CONJUNCTION,s->left,s->right);
                        aux->left = get_fnb(aux->left,1,0);
                        aux->right = get_fnb(aux->right,0,1);
                        
                        return aux;
                    }
                } else {
                    if (theta == 0) { // T: (p then q) = (F:p and T: theta(q)) or (F:theta(p) or T:q)
                        tnode *aux;
                        
                        aux = create_tnode(DISJUNCTION,DISJUNCTION,NULL,NULL);
                        aux->left = create_tnode(CONJUNCTION,CONJUNCTION,s->left,s->right);
                        aux->right = create_tnode(DISJUNCTION,DISJUNCTION,s->left,s->right);
                        
                        aux->left->left = get_fnb(aux->left->left,0,0);
                        aux->left->right = get_fnb(aux->left->right,1,1);
                        aux->right->left = get_fnb(aux->right->left,0,1);
                        aux->right->right = get_fnb(aux->right->right,1,0);
                        
                        return aux;
                    } else { // T: theta(p then q) = F:p or T: theta(q)
                        tnode *aux;
                        
                        aux = create_tnode(DISJUNCTION,DISJUNCTION,s->left,s->right);
                        aux->left = get_fnb(aux->left,0,0);
                        aux->right = get_fnb(aux->right,1,1);
                        
                        return aux;
                    }
                }
            }
            break;
        default:
            {
                printf("Error 2!\n");
            }
            break;
        }
}
