#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "prover.h"
#include "tree.h"
#include "symbol_table.h"

extern tnode *create_tnode(int type, int id, tnode *left, tnode *right);
extern tnode *copy_tree(tnode *s);
extern prop_node *insert_p_node (char *name);
extern void print_prop (int id);
extern tnode *renamelist;
extern tnode *proplist;
extern void print_tree(tnode *s);
extern tnode *free_tree2(tnode *s);
extern int numprops;
extern struct prop_node *propsid;
extern FILE *fp;

int numvariable;
int had_renaming;
int cont;
int numbytes;
char strout[60];

void add_remaining_clauses() {
    prop_node *p, *q;
    for (p = propsid; p != NULL; p = p->hid.next) {
        if((strncmp(p->name,"_",1) != 0) && (strncmp(p->name,"THETA", 5) != 0)) {
            for(q = propsid; q != NULL; q = q->hid.next) {
                if(strncmp(q->name,"THETA",5) == 0) {
                    int len = snprintf(NULL, 0, "THETA(%s)",p->name) + 1;
                    char *name = (char *) malloc(len); 
                    len = snprintf(name, len, "THETA(%s)",p->name);
                    if(strcmp(name,q->name) == 0) {
                        free(name);
                        fprintf(fp,"-%d %d 0\n",p->id,q->id);
                        cont++;
                        break;
                    }
                    free(name);
                }
            }
        }
    }
}

void count_clauses (tnode *s) {
    if (s == NULL)
        printf("Can't count clauses.\n");
    else switch (s->type) {
        case CONJUNCTION:
            {
                if (s->left->type == DISJUNCTION) {
                    cont++;
                    count_clauses(s->left);
                } else {
                    count_clauses(s->left);
                }
                if (s->right->type == DISJUNCTION) {
                    cont++;
                    count_clauses(s->right);
                } else {
                    count_clauses(s->right);
                }
            }
            break;
        case DISJUNCTION:
            {
                if (s->left->id == CTRUE || s->right->id == CTRUE) {
                    cont--;
                }
                count_clauses(s->right);
            }
            break;
        }
}

void export_clauses (tnode *s) {
    if (s == NULL)
        printf("Can't export clauses.\n");
    else switch (s->type) {
        case CONJUNCTION:
            {
                export_clauses(s->left);
                export_clauses(s->right);
            }
            break;
        case DISJUNCTION:
            {
                if (s->left->value == 1) {
                    if (s->left->id != CFALSE && s->left->id != CTRUE) {
                        char strin[10];
                        sprintf(strin,"%d",s->left->id);
                        strcat(strout,strin);
                        
                    } else if (s->left->id == CTRUE) {
                        strcpy(strout,"");
                    }
                } else {
                    if (s->left->id != CFALSE) {
                        char strin[10];
                        sprintf(strin,"-%d",s->left->id);
                        strcat(strout,strin);
                    }
                }
                if (s->right->type == PROP || s->right->type == CONSTANT) {
                    strcat(strout," ");
                    if (s->right->value == 1) {
                        if (s->right->id != CFALSE && s->right->id != CTRUE) {
                            char strin[10];
                            sprintf(strin,"%d",s->right->id);
                            strcat(strout,strin);
                        } else if (s->right->id == CTRUE) {
                            strcpy(strout,"");
                        }
                    } else {
                        if (s->right->id != CFALSE) {
                            char strin[10];
                            sprintf(strin,"-%d",s->right->id);
                            strcat(strout,strin);
                        }
                    }
                    if (strlen(strout) != 0) {
                        strcat(strout," 0\n");
                        fprintf(fp,"%s",strout);
                        strcpy(strout,"");
                        cont++;
                    }
                    
                } else {
                    strcat(strout," ");
                    export_clauses(s->right);
                }
            }
            break;
        }
}

tnode *remove_doubleD(tnode *s) {
    tnode *aux, *temp;
    if (s == NULL)
        return s;
    if (s->type == DISJUNCTION) {
        if (s->left->type == DISJUNCTION) {
            aux = s->left->right;
            s->right = create_tnode(DISJUNCTION,DISJUNCTION,aux,s->right);
            temp = s->left;
            s->left = s->left->left;
            s->left = remove_doubleD(s->left);
            if (s->left->type == DISJUNCTION) {
                s = remove_doubleD(s);
            }
            free(temp);
        }
    }
    s->left = remove_doubleD(s->left);
    s->right = remove_doubleD(s->right);
    return s;
}

void print_clauses(tnode *s) {
    if (s == NULL)
        printf("Can't print clauses.\n");
    else switch (s->type) {
        case CONJUNCTION:
            {
                if (s->left->type == DISJUNCTION) {
                    printf("%d: (", cont);
                    cont++;
                    print_clauses(s->left);
                } else {
                    print_clauses(s->left);
                }
                if (s->right->type == DISJUNCTION) {
                    printf("%d: (", cont);
                    cont++;
                    print_clauses(s->right);
                } else {
                    print_clauses(s->right);
                }
            }
            break;
        case DISJUNCTION:
            {
                if (s->left->value == 1) {
                    printf("T:");
                    print_prop(s->left->id);
                } else {
                    printf("F:");
                    print_prop(s->left->id);
                }
                if ((s->right->type == PROP) || (s->right->type == CONSTANT)) {
                    printf(" || ");
                    if (s->right->value == 1) {
                        printf("T:");
                        print_prop(s->right->id);
                    } else {
                        printf("F:");
                        print_prop(s->right->id);
                    }
                    printf(")\n");
                } else {
                    printf(" || ");
                    print_clauses(s->right);
                }
            }
            break;
        }
}

tnode *remove_impl(tnode *s) {
    if (s == NULL)
        return NULL;
    else switch (s->type) {
        case IMPLICATION:
            {
                s->type = DISJUNCTION;
                s->left->value = 0;
                s->right = remove_impl(s->right);
                return s;
            }
            break;
        default:
            {
                s->left = remove_impl(s->left);
                s->right = remove_impl(s->right);
                return s;
            }
            break;
        }
}

tnode *get_fnf(tnode *s) {
    if (s == NULL)
        return NULL;
    else switch (s->type) {
        case CONSTANT:
            {
                return s;
            }
            break;
        case PROP:
            {
                return s;
            }
            break;
        case DISJUNCTION:
            {
                tnode *aux, *aux1, *aux2, *aux3, *aux4, *aux5;
                prop_node *p1, *p2;
                
                aux2 = NULL;
                aux4 = NULL;
                if (s->left->type == CONJUNCTION) {
                    had_renaming++;
                    int len = snprintf(NULL, 0, "_t%d",numvariable) + 1;
                    char *name = (char *) malloc(len); 
                    len = snprintf(name, len, "_t%d",numvariable);
                    numvariable++;
                    p1 = insert_p_node(name);
                    aux1 = s->left;
                    s->left = create_tnode(PROP,p1->id,NULL,NULL);
                    aux2 = create_tnode(IMPLICATION,IMPLICATION,NULL,aux1);
                    aux2->left = create_tnode(PROP,p1->id,NULL,NULL);
                    aux2 = get_fnf(aux2);
                }
                if (s->right->type == CONJUNCTION) {
                    had_renaming++;
                    int len2 = snprintf(NULL, 0, "_t%d",numvariable) + 1;
                    char *name2 = (char *) malloc(len2); 
                    len2 = snprintf(name2, len2, "_t%d",numvariable);
                    numvariable++;
                    p2 = insert_p_node(name2);
                    aux3 = s->right;
                    s->right = create_tnode(PROP,p2->id,NULL,NULL);
                    aux4 = create_tnode(IMPLICATION,IMPLICATION,NULL,aux3);
                    aux4->left = create_tnode(PROP,p2->id,NULL,NULL);
                    aux4 = get_fnf(aux4);
                }
                if ((aux2 != NULL) && (aux4 != NULL)) {
                    s = create_tnode(CONJUNCTION,CONJUNCTION,s,NULL);
                    s->right = create_tnode(CONJUNCTION,CONJUNCTION,aux2,aux4);
                    return s;
                } else if (aux2 != NULL) {
                    s->right = get_fnf(s->right);
                    if (s->right->type == CONJUNCTION) {
                        aux5 = s->right->right;
                        aux = s->right;
                        s->right = s->right->left;
                        aux2 = create_tnode(CONJUNCTION,CONJUNCTION,aux2,aux5);
                        free(aux);
                    }
                    s = create_tnode(CONJUNCTION,CONJUNCTION,s,aux2);
                    return s;
                } else if (aux4 != NULL) {
                    s->left = get_fnf(s->left);
                    if (s->left->type == CONJUNCTION) {
                        aux5 = s->left->right;
                        aux = s->right;
                        s->right = s->left->left;
                        aux4 = create_tnode(CONJUNCTION,CONJUNCTION,aux4,aux5);
                        free(aux);
                    }
                    s = create_tnode(CONJUNCTION,CONJUNCTION,s,aux4);
                    return s;
                } else {
                    s->left = get_fnf(s->left);
                    s->right = get_fnf(s->right);
                    
                    return s;
                }
            }
            break;
        case IMPLICATION:
            {
                int original_renaming;
                tnode *aux, *aux1, *aux2, *aux3, *aux4;
                
                original_renaming = had_renaming;
                
                if (s->right->type == CONJUNCTION) {
                    aux = s;
                    aux1 = s->right->left;
                    aux2 = s->right->right;
                    aux3 = create_tnode(IMPLICATION,IMPLICATION,NULL,aux1);
                    aux3->left = create_tnode(PROP,s->left->id,NULL,NULL);
                    aux3 = get_fnf(aux3);
                    aux4 = create_tnode(IMPLICATION,IMPLICATION,NULL,aux2);
                    aux4->left = create_tnode(PROP,s->left->id,NULL,NULL);
                    aux4 = get_fnf(aux4);
                    s = create_tnode(CONJUNCTION,CONJUNCTION,aux3,aux4);
                    free(aux->left);
                    free(aux->right);
                    free(aux);
                    return s;
                } else if (s->right->type == IMPLICATION) {
                    aux = s;
                    s = s->right;
                    s = get_fnf(s);
                    
                    free(aux->left);
                    free(aux);
                    return s;
                } else {
                    s->right = get_fnf(s->right);
                    if (original_renaming < had_renaming) {
                        had_renaming--;
                        s = get_fnf(s);
                    }
                    return s;
                }
            }
            break;
        default:
            {
                return s;
            }
            break;
        }
}
