#include <stdlib.h>
#include <stdio.h>
#include "prover.h"
#include "tree.h"

extern tnode *create_tnode(int type, int id, tnode *left, tnode *right);
extern tnode *copy_tree(tnode *s);
extern void print_tree(tnode *s);

tnode *get_nnf(tnode *s) {
    if (s == NULL)
        return NULL;
    else switch (s->type) {
        case CONSTANT:
        {
            return s;
        }
        case PROP:
        {
            return s;
        }
        break;
        case NEGATION:
        {
            tnode *aux, *temp;
            temp = s;
            aux = create_tnode(CONSTANT,CFALSE,NULL,NULL);
            s = create_tnode(IMPLICATION,IMPLICATION,s->left,aux);
            s->left = get_nnf(s->left);
            free(temp);
            return s;
        }
        break;
        case CONJUNCTION:
        { // p and q = neg (neg a or neg b)

            s = create_tnode(NEGATION,NEGATION,s,NULL);
            s->left->type = DISJUNCTION;
            s->left->id = DISJUNCTION;
            s->left->left = create_tnode(NEGATION,NEGATION,s->left->left,NULL);
            s->left->right = create_tnode(NEGATION,NEGATION,s->left->right,NULL);
            s = get_nnf(s);

            return s;
        }
        break;
        case DISJUNCTION:
        { // p or q = (p then q) then q
            tnode *aux;
            
            aux = copy_tree(s->right);
            s->type = IMPLICATION;
            s->id = IMPLICATION;
            s = create_tnode(IMPLICATION,IMPLICATION,s,aux);
            s->left = get_nnf(s->left);
            s->right = get_nnf(s->right);
            
            return s;
        }
        break;
        case IMPLICATION:
        { // p then q
            s->left = get_nnf(s->left);
            s->right = get_nnf(s->right);
            
            return s;
        }
        break;
        case DOUBLEIMP:
        { // p double_then q = (p then q) and (q then p)
            tnode *aux1, *aux2, *aux3;
            
            aux1 = copy_tree(s->left);
            aux2 = copy_tree(s->right);
            aux3 = create_tnode(IMPLICATION,IMPLICATION,aux2,aux1);
            s->type = IMPLICATION;
            s->id = IMPLICATION;
            s = create_tnode(CONJUNCTION,CONJUNCTION,s,aux3);
            s = get_nnf(s);
            
            return s;
        }
        default:
            printf("We have a problem, Houston. In NNF.\n");
            return s;
    }
}
