/* 
 * Parser for modal formulae
*/

%error-verbose
 //%locations
%debug
%defines
%pure-parser
%code requires {
#include "tree.h"
  
  extern tnode *root;
  extern tnode *add_proplist(int id, int value);
  extern tnode *create_tnode(int type, int id, tnode *left, tnode *right);
}


%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "prover.h"
#include "uthash.h"
#include "symbol_table.h"

  /* flex functions */

  extern int yylex () ;
  extern char *yytext;
  extern FILE *yyin;

  /* Max stack size for parsing */
  
#define YYMAXDEPTH 136854775807
    
  /* symbol table functions in symbol_table.c" */

#define MAX(X, Y) (((X) > (Y)) ? (X) : (Y))

  /* prototype for the error function */

  void yyerror (char const *s);
 
  /* global variables from tokenizer.l */

  extern int numtokens;
  extern int numlines;
  extern int numcolumns;

  /* global variables for parser.y */

  int numerrors = 0;  
  int numprops = 2; // 1 and -1 are reserved for CTRUE an CFALSE, 0 is reserved for CSTART
  int inputsize = 0;

  /* global variables for the prover */


  extern prop_node *find_prop (int id);
  extern prop_node *create_p_node (char *name; int id;);
  extern prop_node *insert_p_node (char *name);

  struct prop_node *propsname = NULL;
  struct prop_node *propsid = NULL;    /* important! initialize to NULL */
  struct agent_node *agentsname = NULL; 
  struct agent_node *agentsid = NULL;  
%}

%union{
  char *strValue;
  tnode *tnode;
}

%left TIFF 
%left TIMPLY 
%left TONLYIF 
%left TOR 
%left TAND
%right TNOT 

%token TIFF "double implication"
%token TIMPLY "implication"
%token TONLYIF "only if"
%token TOR "disjunction"
%token TAND "conjunction"
%token TNOT "negation"

%token <strValue> TNAME "identifier"
%token <strValue> TNUMBER "number"
%token <strValue> TTRUE TFALSE  "constant"

%token TSET
%token TDOT "."
%token TCOMMA ","
%token TFORMULAS "formulas"
%token TEND "end of list"
%token TLPAREN "("
%token TRPAREN ")"

%type <tnode> formula
%type <tnode> proposition

%%

begin : {root = NULL;}
         | formula TDOT
	 {
	   root = $1;
	 }
	 | error TDOT {numerrors++;}
         ;

formula : formula TIFF formula 
          { // double-implications are not treated as abbreviations.
            // I would have a dag if I didn't want to copy the whole tree. 
            // and I don't want to deal with polarity at this point. 
	    // I am postponing whatever is to be done to when I need to put the formula into NNF.
	  inputsize += 1;
	  tnode *new = create_tnode(DOUBLEIMP,DOUBLEIMP,$1,$3);
	  $$ = new;
        }
        | formula TIMPLY formula
        {
	  inputsize += 1;
	  tnode *new = create_tnode(IMPLICATION,IMPLICATION,$1,$3);
	  $$ = new;
        }
        | formula TONLYIF formula
        {
	  inputsize += 1;
	  tnode *new = create_tnode(IMPLICATION,IMPLICATION,$3,$1);
	  $$ = new;
        }
        | formula TOR formula
        {
	  inputsize += 1;
	  tnode *new = create_tnode(DISJUNCTION,DISJUNCTION,$1,$3);
	  $$ = new;
        }
        | formula TAND formula
        {
	  inputsize += 1;
	  tnode *new = create_tnode(CONJUNCTION,CONJUNCTION,$1,$3);
	  $$ = new;
        }
        | TNOT formula 
        {
	  // I was eliminating double negations here, but that changes the counting of the input formula. 
	  inputsize += 1;
      tnode *right = create_tnode(CONSTANT,CFALSE,NULL,NULL);
      tnode *new = create_tnode(IMPLICATION,IMPLICATION,$2,right);	
	  $$ = new;
	    }
        | TLPAREN formula TRPAREN
        {
	  $$ = $2;
	    }
        | proposition
	    { 
	  $$ = $1;
        }
        ;


proposition: TNAME
           {
	     inputsize += 1;
         prop_node *p;
         char *pname = strdup($1);

	     char *s = malloc(snprintf(NULL, 0, "%s", pname) + 1);
	     sprintf(s, "%s", pname);
	     
	     p = insert_p_node(s);
	     tnode *new = create_tnode(PROP,p->id,0,0);
	     $$ = new;
	     free(pname);
	     free($1);
	   }
           | TTRUE
	   {
	     inputsize += 1;
	     tnode *new = create_tnode(CONSTANT,CTRUE,NULL,NULL);
	     $$  = new;
	   }
           | TFALSE
	   {
	     inputsize += 1;
	     tnode *new = create_tnode(CONSTANT,CFALSE,NULL,NULL);
	     $$ = new;
	   } 
           ;

%%


void yyerror (char const *s) {
  fprintf (stderr, "\n Error: %s, line %d, column %d\n",s,numlines,numcolumns);
}


