#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include "parser.tab.h"
#include "prover.h"
#include "symbol_table.h"

int numprops;

/* variables in tokeniser.l */

extern FILE *yyin;
extern int yylex_destroy();
extern void yyrestart( FILE *new_file );

extern int numtokens;
extern int numlines;
extern int numcolumns;
extern int inputsize;

/* global variables for parser.y */

extern int yy_scan_string(const char *);

extern int numerrors;
extern int numagents;
extern int numprops;
extern int numbytes;

extern void symbol_table_init(void);

tnode *root; // pointer for the root of the syntatic tree
int newtemp; // number of new propositional symbols

extern tnode *free_tree (tnode *s);
extern tnode *free_tree2(tnode *s);
extern tnode *get_nnf(tnode *s);
extern tnode *get_fnb(tnode *s, int value, int theta);
extern tnode *get_fnf(tnode *s);
extern tnode *remove_impl(tnode *s);
extern tnode *remove_doubleD(tnode *s);
extern int cont;
extern int numvariable;
extern int had_renaming;
extern int numprops;
extern int print_tree(tnode *s);
extern void print_symbols_tables(void);
extern void export_clauses(tnode *s);
extern void count_clauses(tnode *s);
extern void print_clauses(tnode *s);
extern void add_remaining_clauses(void);
extern int count_symbols_tables();

extern void clear_all(void);
extern void clear_hashes(void);
extern char strout[60];

FILE *fp;

int main(int argc, char* argv[]) {
  int oldnumprops;
  int i = 1;
  i = 1;
  
  yyin=fopen(argv[i], "r");
  if (yyin == NULL) {
    printf("\n Input file not found: %s\n",argv[i]);
    return 0;
  }
 
  symbol_table_init();
 
  yyparse();
  fclose(yyin);
  yylex_destroy();
  
  if (numerrors != 0) {
    printf("\n Number of syntactical errors: %d\n",numerrors);
    exit(1);
  }
  else {
    tnode *oldroot, *nextrenamelist, *nextproplist;
    
    cont = 0;
    numvariable = 0;
    had_renaming = 0;
    numbytes = 0;
    strcpy(strout,"");
    oldnumprops = numprops-2;
    root = get_nnf(root);
    oldroot = root;
    root = get_fnb(root, 0, 0);
    free_tree(oldroot);
    root = get_fnf(root);
    root = remove_impl(root);
    root = remove_doubleD(root);

    count_clauses(root);
    fp = fopen("clauses.out","w");
    fprintf(fp,"p cnf %d %d\n",numprops+9,cont+oldnumprops);
    export_clauses(root);
    add_remaining_clauses();
    fclose(fp);
    
    free_tree(root);
    clear_hashes();
    clear_all();
    
    system("minisat clauses.out > temp.out");
    system("grep SATISFIABLE temp.out");
    system("rm temp.out");
    system("rm clauses.out");
  }
  return 0;
}

