#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "uthash.h"
#include "symbol_table.h"
#include "prover.h"

extern int global;
extern int numprops;
extern int newtemp;
extern int numagents;
extern struct prop_node *propsname;
extern struct prop_node *propsid;
extern struct agent_node *agentsname; 
extern struct agent_node *agentsid; 

prop_node *create_p_node (char* name, int id) {
  prop_node *new = (prop_node *) malloc(sizeof(prop_node));
  new->name = name;
  new->id = id;
  new->occur = 1;
  return new;
}


prop_node *insert_p_node (char *name) {
  int id;
  prop_node *p;
  HASH_FIND_STR(propsname,name,p);
  if (p) {
    id = p->id;
    p->occur++;
    free(name);
  }
  else {
    id = 10 + numprops++;
    p = create_p_node(name,id);
    HASH_ADD_KEYPTR(hh,propsname,p->name,strlen(p->name),p);
    HASH_ADD(hid,propsid,id,sizeof(int),p);  
  }
  return p;
}

prop_node *find_prop (int id) {
  prop_node *p;
  HASH_FIND(hid,propsid,&id,sizeof(int),p);
  return p;
}

void print_prop (int id) {
  prop_node *p;
  HASH_FIND(hid,propsid,&id,sizeof(int),p);
  if (p != NULL)
    printf("%s",p->name);
  else printf("problem");
}

void clear_hashes (void) {
  HASH_CLEAR(hh,propsname);
}

void free_pnode(prop_node *p) {
  if (p != NULL) {
    if (p->name != NULL)
      free(p->name);
    free(p);
  }
}

void clear_all (void) {
  prop_node *p,*tmpp;
  
  HASH_ITER(hid,propsid,p,tmpp) {
    HASH_DELETE(hid,propsid,p);
    free_pnode(p);
  }

  HASH_CLEAR(hid,propsid);
  free(propsid);
}

void symbol_table_init(void) {

  prop_node *p;
  int id;

  id = CTRUE;
  int len = snprintf(NULL, 0, "_true") + 1;
  char *s1 = (char *) malloc(len); 
  len = snprintf(s1, len, "_true");
  p = create_p_node(s1,id);
  HASH_ADD(hid,propsid,id,sizeof(int),p);
  
  id = CFALSE;
  len = snprintf(NULL, 0, "_false") + 1;
  char *s2 = (char *) malloc(len); 
  len = snprintf(s2, len, "_false");
  p = create_p_node(s2,id);
  HASH_ADD(hid,propsid,id,sizeof(int),p);
  
  id = CSTART;
  len = snprintf(NULL, 0, "_start") + 1;
  char *s3 = (char *) malloc(len); 
  len = snprintf(s3, len, "_start");
  p = create_p_node(s3,id);
  HASH_ADD(hid,propsid,id,sizeof(int),p);
}

void print_symbols_tables(void) {
  prop_node *p;
 
  for (p = propsid; p != NULL; p = p->hid.next) {
    printf("\n Propositional Symbol: %s, id = %d\n", p->name,p->id);
  }
}

int count_symbols_tables() {
  prop_node *p;
  int num = 0;
 
  for (p = propsid; p != NULL; p = p->hid.next) {
    num++;
  }
  num = num - 3;
  return num;
}
