#include <stdlib.h>
#include <stdio.h>
#include "prover.h"
#include "uthash.h"
#include "symbol_table.h"
#include "tree.h"
#include <signal.h>

#define MAX(X, Y) (((X) > (Y)) ? (X) : (Y))

extern void print_agent (int id);
extern void print_prop (int id);
extern prop_node *find_prop (int id);
extern unsigned int hash_tree (tnode *t);

extern int phase;

tnode *free_tree(tnode *t);

int indentation = 1;
int tab = 4;

tnode *copy_node(tnode *s) {
    tnode *new = malloc(sizeof(tnode));
    new->type = s->type;
    new->id = s->id;
    new->left = s->left;
    new->right = s->right;
    new->value = s->value;
    new->theta = s->theta;
    return new;
}

tnode *create_tnode(int type, int id, tnode *left, tnode *right) {
  tnode *new = malloc(sizeof(tnode));
  new->type = type;
  new->id = id;
  new->left = left;
  new->right = right;
  new->value = 1;
  new->theta = 0;
  return new;
}

tnode *copy_tree(tnode *s) {
  if (s == NULL) return NULL;
  else {
    tnode *root = malloc(sizeof(tnode));
    root->type = s->type;
    root->id = s->id;
    root->value = s->value;
    root->theta = s->theta;
    root->left = copy_tree(s->left);
    root->right = copy_tree(s->right);
    return root;
  }
}

int print_tree(tnode *s) {  
  int i;
  int size1, size2;
  if (s!=NULL) {
    for (i=0;i<indentation;i++) printf(" ");
  
    switch (s->type) {
    case CONSTANT:
      if (s->id == CTRUE) {
         printf("_true\n");
      } else if (s->id == CFALSE) {
         printf("_false\n");
      } else if (s->id == CSTART) {
         printf("_start\n");
      }
      return 1;
      break;
    case PROP: 
      printf("PROPOSITION: ");
      if (s->value == 1) {
        printf("T:");
        print_prop(s->id);
      } else {
        printf("F:");
        print_prop(s->id);
      }
      printf("\n"); 
      return 1;
      break;
    case NEGATION: 
      printf("NEGATION");
      printf("\n"); 
      indentation += tab;
      size1 = print_tree(s->left);
      indentation -= tab;
      return size1 + 1;
      break;
    case CONJUNCTION:
      printf("CONJUNCTION");
      printf("\n"); 
      indentation += tab;
      size1 = print_tree(s->left);
      size1 += print_tree(s->right);

      indentation -= tab;
      return size1 + 1;
      break;
    case DISJUNCTION:
      printf("DISJUNCTION");
      printf("\n"); 
      indentation += tab;
      size1 = print_tree(s->left);
      size1 += print_tree(s->right);
      indentation -= tab;
      return size1 + 1;
      break;
    case IMPLICATION:
      printf("IMPLICATION");
      printf("\n"); 
      indentation += tab;
      size1 = print_tree(s->left);
      size2 = print_tree(s->right);
      indentation -= tab;
      return size1 + size2 + 1;
      break;
    case DOUBLEIMP:
      printf("DOUBLE IMPLICATION");
      printf("\n"); 
      indentation += tab;
      size1 = print_tree(s->left);
      size2 = print_tree(s->right);
      indentation -= tab;
      return size1 + size2 + 1;
      break;
    default:
      printf("We have a problem, Houston. Printing the tree.\n");
    }
  }
  return 0;
}

tnode *free_tree(tnode *t) {
  if (t != NULL) {
    if (t->left != NULL)
      t->left = free_tree(t->left);
    if (t->right != NULL)
      t->right = free_tree(t->right);
    free(t);
    t = NULL;
  }
  return t;
}

tnode *free_tree2(tnode *t) {
  if (t != NULL) {
    if (t->left != NULL)
        if (t->left->type != PROP && t->left->type != CONSTANT)
            t->left = free_tree2(t->left);
    if (t->right != NULL)
        if (t->right->type != PROP && t->right->type != CONSTANT)
            t->right = free_tree2(t->right);
    free(t);
    t = NULL;
  }
  return t;
}


